﻿Public Class Form1

    Dim dongle As New SmartDongle()
    Public P1 As Int64 = &HEC6CC589AEFD1E75
    Public P2 As Int64 = &HFCEC0A6A82747B3F

    Private Sub getSerialNumber_Click(sender As System.Object, e As System.EventArgs) Handles getSerialNumber.Click
        Dim sn(SmartDongle.CP2_SERIALNUM_LENGTH) As Char
        Dim dnglError As SmartDongle.SmartDnglError

        dnglError = dongle.GetSerialNumber(sn)
        Dim serialNumberString As String = New String(sn, 0, sn.Length)
        If dnglError = SmartDongle.SmartDnglError.UskOk Then
            serialNumber.Text = sn
        Else
            smartDongleMessages.Text = "Error reading SmartDongle serial number."
        End If
    End Sub

    Private Sub ledOff_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ledOff.CheckedChanged
        dongle.Off()
    End Sub

    Private Sub ledRed_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ledRed.CheckedChanged
        dongle.Red()
    End Sub

    Private Sub ledGreen_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles ledGreen.CheckedChanged
        dongle.Green()
    End Sub

    Private Sub checkValid_Click(sender As System.Object, e As System.EventArgs) Handles checkValid.Click
        smartDongleData.Text = ""
        dongle.Red()
        ledRed.Checked = True

        ' Reading the SmartDongle at address 0 with null buffer and 0 size
        ' to check for existence instead of actually reading.
        Dim dnglError As SmartDongle.SmartDnglError
        Dim tempP1 As Int64
        Dim tempP2 As Int64
        Dim start As DateTime
        Dim buf() As Byte = New Byte(0) {}

        If (Not diffKeys.Checked) Then
            start = DateTime.Now
            dnglError = dongle.Read(P1, P2, 0, buf, 0)
        Else
            Try
                tempP1 = Convert.ToUInt64(tempKey1.Text, 16)
                tempP2 = Convert.ToUInt64(tempKey2.Text, 16)
            Catch ex As Exception
                MessageBox.Show("Invalid Key Format." & vbNewLine & ex.ToString())
                Return
            End Try
            start = DateTime.Now
            dnglError = dongle.Read(tempP1, tempP2, 0, buf, 0)
        End If

        Dim elapsed As TimeSpan = DateTime.Now - start
        dongle.Green()
        ledGreen.Checked = True
        If (dnglError = 0) Then
            If (timeFunctions.Checked) Then
                smartDongleData.Text = "Check completed in " & elapsed.TotalSeconds & " sec"
            End If
            smartDongleMessages.Text = "Valid SmartDongle Found."
        Else
            smartDongleMessages.Text = SmartDongle.GetErrorString(dnglError)
        End If
    End Sub

    Private Sub readData_Click(sender As System.Object, e As System.EventArgs) Handles readData.Click
        Dim dataError As Integer = 0
        Dim addr As UInt16 = 0
        Dim numBytes As Integer = 0

        ' Ensure the address is a valid number within range.
        Try
            addr = Convert.ToUInt16(readWriteAddress.Text)
            If (addr > 32431) Then
                smartDongleData.Text = "Overflow Exception: Address value must be between 0 and 32431"
                dataError = 1
                Return
            End If
        Catch ex1 As FormatException
            smartDongleData.Text = "Format Exception: Address value must be a valid number between 0 and 32431"
            dataError = 1
            Return
        Catch ex2 As OverflowException
            smartDongleData.Text = "Overflow Exception: Address value must be between 0 and 32431"
            dataError = 1
            Return
        End Try

        ' Ensure the number of bytes is a valid number within range.
        Try
            numBytes = Convert.ToInt32(numBytesRead.Text)
            If ((addr + numBytes - 1) > 32431) Then
                smartDongleData.Text = "Overflow Exception: (Address + Number of Bytes - 1) must be <= 32431"
                dataError = 1
                Return
            End If
        Catch ex1 As FormatException
            smartDongleData.Text = "Format Exception: Number of Bytes value must be a valid number between 0 and 32431"
            dataError = 1
            Return
        Catch ex2 As OverflowException
            smartDongleData.Text = "Overflow Exception: Number of Bytes value must be between 0 and 32431"
            dataError = 1
            Return
        End Try

        ' If address and number of bytes are valid.
        If (dataError = 0) Then
            Dim buff() As Byte = New Byte(numBytes) {}

            dongle.Red()
            ledRed.Checked = True
            Dim dnglError As SmartDongle.SmartDnglError
            Dim tempP1 As Int64
            Dim tempP2 As Int64
            Dim start As DateTime

            ' Read SmartDongle at given address for given number of bytes.
            If (Not diffKeys.Checked) Then
                start = DateTime.Now
                dnglError = dongle.Read(P1, P2, addr, buff, numBytes)
            Else
                Try
                    tempP1 = Convert.ToUInt64(tempKey1.Text, 16)
                    tempP2 = Convert.ToUInt64(tempKey2.Text, 16)
                Catch ex As Exception
                    MessageBox.Show("Invalid Key Format." & vbNewLine & ex.ToString())
                    Return
                End Try
                start = DateTime.Now
                dnglError = dongle.Read(tempP1, tempP2, addr, buff, numBytes)
            End If

            Dim elapsed As TimeSpan = DateTime.Now - start
            dongle.Green()
            ledGreen.Checked = True

            ' If error reading the SmartDongle, return error message.
            If (dnglError <> SmartDongle.SmartDnglError.UskOk) Then
                smartDongleMessages.Text = SmartDongle.GetErrorString(dnglError)
                smartDongleData.Text = ""
                ' Else if no error, return data read from SmartDongle.
            Else
                smartDongleMessages.Text = ""
                If (timeFunctions.Checked) Then
                    smartDongleData.Text = "Read completed in " & elapsed.TotalSeconds & " sec" & vbNewLine & System.Text.Encoding.UTF8.GetString(buff)
                Else
                    smartDongleData.Text = System.Text.Encoding.UTF8.GetString(buff)
                End If
            End If
        End If
    End Sub

    Private Sub writeData_Click(sender As System.Object, e As System.EventArgs) Handles writeData.Click
        Dim dataError As Integer = 0
        Dim addr As UInt16 = 0
        ' Ensure the entered address is a valid number within range.
        Try
            addr = Convert.ToUInt16(readWriteAddress.Text)
            If (addr > 32431) Then
                smartDongleData.Text = "Overflow Exception: Address value must be between 0 and 32431"
                dataError = 1
            End If
        Catch ex1 As FormatException
            smartDongleData.Text = "Format Exception: Address value must be a valid number between 0 and 32431"
            dataError = 1
        Catch ex2 As OverflowException
            smartDongleData.Text = "Overflow Exception: Address value must be between 0 and 32431"
            dataError = 1
        End Try

        ' Get the number of bytes to be written and ensure
        ' it is not too large for the given address.
        Dim numBytes As Integer = smartDongleData.Text.Length
        If ((addr + numBytes - 1) > 32431) Then
            smartDongleData.Text = "Overflow Exception: (Address + Number of Bytes (characters) to Write - 1) must be <= 32431"
            dataError = 1
        End If

        ' If address and number of bytes are valid.
        If (dataError = 0) Then
            ' Copy data to write to a byte buffer.
            Dim writeBuff() As Byte = System.Text.Encoding.UTF8.GetBytes(smartDongleData.Text)
            Dim readBuff(numBytes - 1) As Byte

            dongle.Red()
            ledRed.Checked = True
            Dim dnglError As SmartDongle.SmartDnglError
            Dim tempP1 As Int64 = 0
            Dim tempP2 As Int64 = 0
            Dim start As DateTime

            ' Write the given data to the given address on the SmartDongle.
            If (Not diffKeys.Checked) Then
                start = DateTime.Now
                dnglError = dongle.Write(P1, P2, addr, writeBuff, numBytes)
            Else
                Try
                    tempP1 = Convert.ToUInt64(tempKey1.Text, 16)
                    tempP2 = Convert.ToUInt64(tempKey2.Text, 16)
                Catch ex As Exception
                    MessageBox.Show("Invalid Key Format." & vbNewLine & ex.ToString())
                    Return
                End Try
                start = DateTime.Now
                dnglError = dongle.Write(tempP1, tempP2, addr, writeBuff, numBytes)
           end if

            Dim elapsed As TimeSpan = DateTime.Now - start
            dongle.Green()
            ledGreen.Checked = True

            ' If there was an error writing to the SmartDongle,
            ' return the error message.
            If (dnglError <> 0) Then
                smartDongleMessages.Text = SmartDongle.GetErrorString(dnglError)
                smartDongleData.Text = ""
            ' Else if there was no write error, read the data back
            ' from the SmartDongle and check if it matches what was
            ' supposed to be written.
            Else
                smartDongleMessages.Text = ""

                dongle.Red()
                ledRed.Checked = True

                ' Read the SmartDongle where the data was just written to.
                If (Not diffKeys.Checked) Then
                    dnglError = dongle.Read(P1, P2, addr, readBuff, numBytes)
                Else
                    dnglError = dongle.Read(tempP1, tempP2, addr, readBuff, numBytes)
                End If
                dongle.Green()

                ' If there is an error reading the SmartDongle, return
                ' the error message.
                ledGreen.Checked = True
                If (dnglError <> 0) Then
                    smartDongleMessages.Text = SmartDongle.GetErrorString(dnglError)
                    smartDongleData.Text = ""
                    ' Else if no error, compare the written and read string and
                    ' return a match/no match message along with the data that
                    ' was read from the SmartDongle.

                Else
                    smartDongleMessages.Text = ""
                    Dim writeString As String = System.Text.Encoding.UTF8.GetString(writeBuff)
                    Dim readString As String = System.Text.Encoding.UTF8.GetString(readBuff)
                    If (writeString = readString) Then
                        readString = "Data Written Matches Data Read." & vbNewLine & "Data Read:" & vbNewLine & readString
                    Else
                        readString = "Data Written Does Not Match Data Read." & vbNewLine & "Data Read:" & vbNewLine & readString
                    End If
                    numBytesRead.Text = numBytes.ToString()
                    If (timeFunctions.Checked) Then
                        readString = "Data written in " & elapsed.TotalSeconds & " sec" & vbNewLine & readString
                    End If
                    smartDongleData.Text = readString
                End If
            End If
        End If
    End Sub
End Class
